from .abstractions import AContext, ANetworkStrategy


class Context(AContext):
    def __init__(self, strategy: ANetworkStrategy):
        self.strategy = strategy

    def request(self) -> ANetworkStrategy:
        return self.strategy



from django.dispatch import receiver
from .models import Organization, OrganizationKeys, UserProxy
from django.db.models.signals import post_save
from mnemonic import Mnemonic
from eth_account import Account
from organizations.tasks import create_organization_did
from django.conf import settings
from django.contrib.auth.forms import PasswordResetForm
from django.http.request import HttpRequest


@receiver(post_save, sender=UserProxy)
def post_save_profile(sender, instance, **kwargs):
    if kwargs.get("created"):
        instance.is_staff = True
        instance.save()
        form = PasswordResetForm({"email": instance.email})
        if form.is_valid():
            request = HttpRequest()
            request.META["SERVER_NAME"] = settings.SERVER_NAME
            request.META["SERVER_PORT"] = settings.SERVER_PORT
            form.save(
                request=request,
                use_https=settings.USE_HTTPS,
                from_email=settings.DEFAULT_FROM_EMAIL,
                html_email_template_name="organizations/welcome_operator.html",
            )


@receiver(post_save, sender=Organization)
def post_save_organization(sender, instance, **kwargs):
    if kwargs.get("created"):
        mnemo = Mnemonic("english")
        words = mnemo.generate(strength=256)
        mnemo.to_seed(words)
        mnemo.to_entropy(words)
        new_organization_acct = Account.create(words)
        organization_keys = OrganizationKeys(
            address=new_organization_acct.address,
            private_key=new_organization_acct.key.hex(),
            public_key=new_organization_acct._key_obj.public_key,
            mnemonic=words,
        )
        organization_keys.save()
        instance.keys = organization_keys
        instance.save()
        create_organization_did.delay(instance.pk)



from .abstractions import ANetworkStrategy
from alastria_service_client.client import AClient as AlastriaAClinet, Client as AlastriaClient
from .models import FactoryArgsModel
from network_service_client.enums import ContractsNames
from alastria_service_client.validators import (
    Address,
    NetworkValidator,
    OnlyNetworkValidator,
    SignatureValidator,
    RunRawTransaction,
    PrepareIDValidator,
    DelegateCallValidator,
    CreateAlastriaIdentityValidator,
    AddKeyValidator,
)
from django.conf import settings


class AlastriaNetworkStrategy(ANetworkStrategy):
    def __init__(self, props: FactoryArgsModel):
        self.alastria_service_client: AlastriaAClinet = AlastriaClient(
            service_host=settings.ALASTRIA_SERVICE_HOST
        )
        self.props = props
        network_data = self.props.net
        self.network_body = NetworkValidator(
            provider=network_data.node["path"],
            identity_manager_contract_address=list(
                filter(
                    lambda contract: contract["name"]
                    == ContractsNames.AlastriaIdentityManager.value,
                    network_data.contracts,
                )
            )[0]["address"],
            identity_manager_contract_abi=list(
                filter(
                    lambda contract: contract["name"]
                    == ContractsNames.AlastriaIdentityManager.value,
                    network_data.contracts,
                )
            )[0]["abi"],
            public_key_registry_contract_address=list(
                filter(
                    lambda contract: contract["name"]
                    == ContractsNames.AlastriaPublicKeyRegistry.value,
                    network_data.contracts,
                )
            )[0]["address"],
            public_key_registry_contract_abi=list(
                filter(
                    lambda contract: contract["name"]
                    == ContractsNames.AlastriaPublicKeyRegistry.value,
                    network_data.contracts,
                )
            )[0]["abi"],
            credential_registry_contract_abi=list(
                filter(
                    lambda contract: contract["name"]
                    == ContractsNames.AlastriaCredentialRegistry.value,
                    network_data.contracts,
                )
            )[0]["abi"],
            credential_registry_contract_address=list(
                filter(
                    lambda contract: contract["name"]
                    == ContractsNames.AlastriaCredentialRegistry.value,
                    network_data.contracts,
                )
            )[0]["address"],
            chainId=network_data.chain_id,
        )

    def create_did(
        self,
        sign_address: Address = Address(""),
        issuer_address: Address = Address(""),
        public_key: str = "",
        issuer_private_key: str = "",
        new_issuer_private_key: str = "",
    ) -> str:
        self.alastria_service_client.run_raw_transaction(
            RunRawTransaction(
                raw_transaction=self.alastria_service_client.signature(
                    SignatureValidator(
                        transaction=self.alastria_service_client.delegate_call(
                            DelegateCallValidator(
                                data=self.alastria_service_client.prepare_alastria_id_encode_abi(
                                    PrepareIDValidator(
                                        sign_address=sign_address, network=self.network_body
                                    )
                                ).response,
                                issuer_address=issuer_address,
                                network=self.network_body,
                            ),
                        ).response,
                        private_key=issuer_private_key,
                        network=self.network_body,
                    )
                ).response,
                network=self.network_body,
            )
        )
        self.alastria_service_client.run_raw_transaction(
            RunRawTransaction(
                raw_transaction=self.alastria_service_client.signature(
                    SignatureValidator(
                        transaction=self.alastria_service_client.create_alastria_identity(
                            CreateAlastriaIdentityValidator(
                                add_public_key_call_data=self.alastria_service_client.add_key(
                                    AddKeyValidator(
                                        public_key=public_key,
                                        network=self.network_body,
                                    )
                                ).response,
                                issuer_address=sign_address,
                                network=self.network_body,
                            )
                        ).response,
                        private_key=new_issuer_private_key.encode("utf-8").strip(),
                        network=self.network_body,
                    )
                ).response,
                network=self.network_body,
            )
        )

        return (
            self.props.net.did_prefix
            + self.alastria_service_client.identity_keys(
                sign_address, OnlyNetworkValidator(network=self.network_body)
            ).response[2:]
        )


class LacchainNetworkStrategy(ANetworkStrategy):
    def __init__(self, props: FactoryArgsModel):
        self.props = props
        self.prefix = props.net.did_prefix

    def create_did(
        self,
        signAddress: Address = Address(""),
        issuer_address: Address = Address(""),
        public_key: str = "",
        issuer_private_key: str = "",
        new_issuer_private_key: str = "",
    ) -> str:

        # DID
        did = self.prefix + issuer_address
        return did



from django.contrib import admin
from .models import Intermediary, Organization, OrganizationDID, OrganizationKeys, Issuer, UserProxy


class IntermediaryAdmin(admin.ModelAdmin):
    model = Intermediary


class OrganizationAdmin(admin.ModelAdmin):
    model = Organization


class OrganizationDIDAdmin(admin.ModelAdmin):
    model = OrganizationDID


class OrganizationKeysAdmin(admin.ModelAdmin):
    model = OrganizationKeys


class IssuerAdmin(admin.ModelAdmin):
    model = Issuer


class UserProxyAdmin(admin.ModelAdmin):
    model = UserProxy
    fields = ("username", "first_name", "last_name", "email")


admin.site.register(Intermediary, IntermediaryAdmin)
admin.site.register(Organization, OrganizationAdmin)
admin.site.register(OrganizationDID, OrganizationDIDAdmin)
admin.site.register(OrganizationKeys, OrganizationKeysAdmin)
admin.site.register(Issuer, IssuerAdmin)
admin.site.register(UserProxy, UserProxyAdmin)



from enum import Enum


class NetworksNames(str, Enum):
    AlastriaDefaultName = "Alastria"
    LacchainDefaultName = "Lacchain"

    @classmethod
    def choices(cls):
        return tuple((i.name, i.value) for i in cls)



# Generated by Django 4.1.3 on 2023-01-20 10:26

from django.db import migrations
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('organizations', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='organization',
            name='networks',
            field=multiselectfield.db.fields.MultiSelectField(choices=[('AlastriaDefaultName', 'Alastria'), ('LacchainDefaultName', 'Lacchain')], default=1, max_length=19),
            preserve_default=False,
        ),
    ]



# Generated by Django 4.1.3 on 2023-01-18 15:06

import django.contrib.auth.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('email', models.EmailField(max_length=254)),
                ('phone', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='OrganizationKeys',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('address', models.CharField(max_length=100)),
                ('private_key', models.CharField(max_length=500)),
                ('public_key', models.CharField(max_length=500)),
                ('mnemonic', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='UserProxy',
            fields=[
            ],
            options={
                'verbose_name': 'Operator',
                'verbose_name_plural': 'Operators',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('auth.user',),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='OrganizationDID',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('network_name', models.CharField(max_length=2500)),
                ('did', models.CharField(max_length=255)),
                ('organization', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='organizations.organization')),
            ],
        ),
        migrations.AddField(
            model_name='organization',
            name='keys',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='organizations.organizationkeys'),
        ),
        migrations.AddField(
            model_name='organization',
            name='operators',
            field=models.ManyToManyField(to='organizations.userproxy'),
        ),
        migrations.CreateModel(
            name='Issuer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True)),
                ('organization', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='organizations.organization')),
            ],
        ),
        migrations.CreateModel(
            name='Intermediary',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('active', models.BooleanField(default=True)),
                ('organization', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='organizations.organization')),
            ],
        ),
    ]
